program BINDconf;

{$APPTYPE CONSOLE}

uses
  WinInet,
  Windows,
  SysUtils,
  ShellAPI,
  Registry,
  FileCtrl;

function DownloadFile(const AURL : String; const AFileName : String) : Boolean;
var
  LAborted      : Boolean;
  LBuffer       : array[0..1023] of Char;
  LBytesRead    : LongWord;
  LBytesWritten : LongInt;
  LNetHandle    : HINTERNET;
  LUrlHandle    : HINTERNET;
  LWriteFile    : File of Char;
begin
  Result := false;

  LNetHandle := InternetOpen('BINDconf', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  if (LNetHandle <> nil) then
  begin
    try
      LUrlHandle := InternetOpenUrl(LNetHandle, PChar(AURL), nil, 0, INTERNET_FLAG_RELOAD, 0);
      if (LUrlHandle <> nil) then
      begin
        try
          LAborted := false;

          AssignFile(LWriteFile, AFileName);
          Rewrite(LWriteFile);
          try
            repeat
              InternetReadFile(LUrlHandle, @LBuffer, SizeOf(LBuffer), LBytesRead);

              if (LBytesRead > 0) then
              begin
                BlockWrite(LWriteFile, LBuffer, LBytesRead, LBytesWritten);
                LAborted := (LBytesRead <> LBytesWritten);
              end;
            until ((LBytesRead = 0) or LAborted);
          finally
            CloseFile(LWriteFile);
          end;

          if not(LAborted) then
            Result := FileExists(AFileName);
        finally
          InternetCloseHandle(LUrlHandle);
        end;
      end;
    finally
      InternetCloseHandle(LNetHandle);
    end;
  end;
end;

procedure ExecuteProgram(const AFileName : String; const AParameters : String);
var
   LExitCode : LongWord;
   LSEInfo   : TShellExecuteInfo;
begin
  if FileExists(AFileName) then
  begin
    FillChar(LSEInfo, SizeOf(LSEInfo), 0);
    LSEInfo.cbSize       := SizeOf(TShellExecuteInfo);
    LSEInfo.fMask        := SEE_MASK_NOCLOSEPROCESS;
    LSEInfo.lpDirectory  := PChar(ExtractFilePath(AFileName));
    LSEInfo.lpFile       := PChar(AFileName);
    LSEInfo.lpParameters := PChar(AParameters);
    LSEInfo.nShow        := SW_SHOWNORMAL;
    LSEInfo.Wnd          := 0;

    if ShellExecuteEx(@LSEInfo) then
    begin
      repeat
        Sleep(100);
        
        GetExitCodeProcess(LSEInfo.hProcess, LExitCode);
      until (LExitCode <> STILL_ACTIVE);
    end;
  end;
end;

function StripBackslash(const APath : String) : String;
const
  CBackslash = '\';
begin
  Result := APath;
  if (Result[Length(Result)] = CBackSlash) then
    Delete(Result, Length(Result), 1);
end;

procedure ConfigureBIND;
const
  CBackslash = '\';
  CBinPath   = 'bin\';
  CConfGen   = 'rndc-confgen.exe';
  CEtcPath   = 'etc\';
  CKeyName   = 'rndc-key';
  CNamedConf = 'named.conf';
  CNamedRoot = 'named.root';
  CPortNum   = '953';
  CRegPath   = '\Software\ISC\BIND\';
  CRegValue  = 'InstallDir';
  CRndc      = 'rndc.exe';
  CRndcConf  = 'rndc.conf';
  CRndcKey   = 'rndc.key';
  CRootURL   = 'http://www.internic.net/zones/named.root';
  CServerIP  = '127.0.0.1';
var
  LBindPath  : String;
  LIndex     : LongInt;
  LKeyFile   : array of String;
  LNextLine  : String;
  LReadFile  : TextFile;
  LRegistry  : TRegistry;
  LWriteFile : TextFile;
begin
  LBindPath := '';

  LRegistry := TRegistry.Create(KEY_READ);
  try
    LRegistry.RootKey := HKEY_LOCAL_MACHINE;

    if LRegistry.OpenKey(CRegPath, false) then
    begin
      try
        LBindPath := LRegistry.ReadString(CRegValue);
        if (LBindPath[Length(LBindPath)] <> CBackslash) then
          LBindPath := LBindPath + CBackslash;
      finally
        LRegistry.CloseKey;
      end;
    end;
  finally
    LRegistry.Free;
  end;

  if DirectoryExists(LBindPath) then
  begin
    if DirectoryExists(LBindPath + CBinPath) then
    begin
      if FileExists(LBindPath + CBinPath + CRndc) then
      begin
        if FileExists(LBindPath + CBinPath + CConfGen) then
        begin
          if ForceDirectories(LBindPath + CEtcPath) then
          begin
            WriteLn('Downloading BIND named.root ...');
            if DownloadFile(CRootURL, LBindPath + CEtcPath + CNamedRoot) then
            begin
              WriteLn('Creating BIND rndc.key ...');
              ExecuteProgram(LBindPath + CBinPath + CConfGen,
                             '-a ' +
                             '-b "512" ' +
                             '-c "' + LBindPath + CEtcPath + CRndcKey + '" ' +
                             '-k "' + CKeyName + '" ' +
                             '-p "' + CPortNum + '" ' +
                             '-s "' + CServerIP + '"');

              if FileExists(LBindPath + CEtcPath + CRndcKey) then
              begin
                SetLength(LKeyFile, 0);

                WriteLn('Reading BIND rndc.key ...');
                try
                  AssignFile(LReadFile, LBindPath + CEtcPath + CRndcKey);
                  Reset(LReadFile);
                  try
                    while not(EoF(LReadFile)) do
                    begin
                      ReadLn(LReadFile, LNextLine);

                      SetLength(LKeyFile, Succ(Length(LKeyFile)));
                      LKeyFile[High(LKeyFile)] := LNextLine
                    end;
                  finally
                    CloseFile(LReadFile);
                  end;
                finally
                  WriteLn('Deleting BIND rndc.key ...');
                  DeleteFile(LBindPath + CEtcPath + CRndcKey);
                end;

                WriteLn('Writing BIND rndc.conf ...');
                AssignFile(LWriteFile, LBindPath + CEtcPath + CRndcConf);
                Rewrite(LWriteFile);
                try
                  for LIndex := 0 to Pred(Length(LKeyFile)) do
                    WriteLn(LWriteFile, LKeyFile[LIndex]);
                  WriteLn(LWriteFile, '');

                  WriteLn(LWriteFile, 'options {');
                  WriteLn(LWriteFile, #$09, 'default-key "', CKeyName, '";');
                  WriteLn(LWriteFile, #$09, 'default-server ', CServerIP, ';');
                  WriteLn(LWriteFile, #$09, 'default-port ', CPortNum, ';');
                  WriteLn(LWriteFile, '};');
                finally
                  CloseFile(LWriteFile);
                end;

                if FileExists(LBindPath + CEtcPath + CRndcConf) then
                begin
                  WriteLn('Writing BIND named.conf ...');
                  AssignFile(LWriteFile, LBindPath + CEtcPath + CNamedConf);
                  Rewrite(LWriteFile);
                  try
                    WriteLn(LWriteFile, 'options {');
                    WriteLn(LWriteFile, #$09, 'directory "', StripBackslash(LBindPath + CEtcPath), '";');
                    WriteLn(LWriteFile, '');
                    WriteLn(LWriteFile, #$09, 'allow-query { ', CServerIP, '; };');
                    WriteLn(LWriteFile, #$09, 'allow-recursion { ', CServerIP, '; };');
                    WriteLn(LWriteFile, '};');
                    WriteLn(LWriteFile, '');
                    WriteLn(LWriteFile, 'zone "." {');
                    WriteLn(LWriteFile, #$09, 'type hint;');
                    WriteLn(LWriteFile, #$09, 'file "', LBindPath, CEtcPath, CNamedRoot, '";');
                    WriteLn(LWriteFile, '};');
                    WriteLn(LWriteFile, '');

                    for LIndex := 0 to Pred(Length(LKeyFile)) do
                      WriteLn(LWriteFile, LKeyFile[LIndex]);
                    WriteLn(LWriteFile, '');

                    WriteLn(LWriteFile, 'controls {');
                    WriteLn(LWriteFile, #$09, 'inet ', CServerIP, ' port ', CPortNum, ' allow { ', CServerIP, '; } keys { "', CKeyName, '"; };');
                    WriteLn(LWriteFile, '};');
                  finally
                    CloseFile(LWriteFile);
                  end;

                  if FileExists(LBindPath + CEtcPath + CNamedConf) then
                  begin
                    WriteLn('Reloading BIND configuration ...');
                    ExecuteProgram(LBindPath + CBinPath + CRndc,
                                   'reload');
                  end
                  else
                    WriteLn('BIND named.conf not written!');
                end
                else
                  WriteLn('BIND rndc.conf not written!');
              end
              else
                WriteLn('BIND rndc.key not created!');
            end
            else
              WriteLn('BIND named.root not downloaded!');
          end
          else
            WriteLn('BIND etc-folder not found!');
        end
        else
          WriteLn('BIND rndc-confgen.exe not found!');
      end
      else
        WriteLn('BIND rndc.exe not found!');
    end
    else
      WriteLn('BIND bin-folder not found!');
  end
  else
    WriteLn('BIND folder not found!');
end;

begin
  WriteLn('BINDconf 0.1b1');
  WriteLn('(C) 2009 coltishWARE');
  WriteLn('');

  ConfigureBIND;
end.
